//LCAO calculator for Wavefunctions of Singly-Ionized Molecular Hydrogen
//Author: Peter Behroozi
//Creation date: 3/24/08
//Compilation Procedure (requires gcc and perl): make
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define VC 14.3996439 // q^2/(4pi e_0) in eV*Angstrom
#define HC 3.80998081 // hbar^2/2m_e in eV*Angstrom^2
#define NATURAL_LENGTH_UNIT (HC/VC) //About 0.264588544 Angstrom
#define NATURAL_ENERGY_UNIT (VC*VC/HC) //About 54.4227793 electron volts
//In these natural units, the Schrodinger equation is simply
// dPsi/d(it) = -(Laplacian) Psi + V Psi, and
// V = -1/r1 - 1/r2
#define A 2.0       //In natural length units
#define E_0 (0.25)  //In natural energy units

//Legendre Polynomials
inline double p1(double ct) {
  return (ct);
}
inline double p2(double ct) {
  return (3*ct*ct - 1);
}
inline double p3(double ct) {
  return (5*ct*ct*ct - 3*ct);
}

//(Denormalized) Wavefunctions of atomic hydrogen
double r10(double r, double z) {
  return (exp(-r/A));
}
double r20(double r, double z) {
  double ra = r/(2.0*A);
  return ((1.0 - ra)*exp(-ra));
}
double r21(double r, double z) {
  double ra = r/(2.0*A);
  return (ra*exp(-ra)*p1(z/r));
}
double r30(double r, double z) {
  double ra = r/(3.0*A);
  return ((1.0 - 2.0*ra + 2.0/3.0*ra*ra)*exp(-ra));
}
double r31(double r, double z) {
  double ra = r/(3.0*A);
  return ((1.0 - ra/2.0)*ra*exp(-ra)*p1(z/r));
}
double r32(double r, double z) {
  double ra = r/(3.0*A);
  return (ra*ra*exp(-ra)*p2(z/r));
}
double r40(double r, double z) {
  double ra = r/(4.0*A);
  return ((1.0 - 3.0*ra + 2.0*ra*ra - ra*ra*ra/3.0)*exp(-ra));
}
double r42(double r, double z) {
  double ra = r/(4.0*A);
  return ((1.0 - ra/3.0)*ra*ra*exp(-ra)*p2(z/r));
}
double r41(double r, double z) {
  double ra = r/(4.0*A);
  return ((1.0 - ra + ra*ra/5.0)*ra*exp(-ra)*p1(z/r));
}
double r43(double r, double z) {
  double ra = r/(4.0*A);
  return (ra*ra*ra*exp(-ra)*p3(z/r));
}

#define WAVEFUNCTIONS 10
double energies[] = {-E_0, -E_0/4.0, -E_0/4.0, -E_0/9.0, -E_0/9.0, -E_0/9.0,
                     -E_0/16.0, -E_0/16.0, -E_0/16.0, -E_0/16.0};
typedef double (*wfptr)(double, double);
wfptr wfuncs[] = {r10, r20, r21, r30, r31, r32, r40, r41, r42, r43};
double coeffs[] = {0,0,0,0,0,0,0,0,0,0};
double d;

#define Z_STEPS 200.0
#define Z_MAX 20.0
#define DZ (Z_MAX/Z_STEPS)
#define R_STEPS 400.0
#define R_MAX 20.0
#define DR (R_MAX/R_STEPS)

//Calculate <psi|H|psi>
double expected_h(void) {  
  int i,j;
  double r, z;
  double da = d/2.0;
  double r1, z1, r2, z2;
  double pp = 0;  //<psi|psi>
  double php = 0; //<psi|h|psi>
  double psi,hpsi,tpsi1, tpsi2;

  for (z=-Z_MAX; z<Z_MAX; z+=DZ) {
    for (r=DR; r<R_MAX; r+=DR) {
      z1 =  da + z;
      z2 =  da - z;
      r1 = sqrt(z1*z1+r*r);
      r2 = sqrt(z2*z2+r*r);
      hpsi = psi = 0;
      for (i=0; i<WAVEFUNCTIONS; i++) {
	tpsi1 = wfuncs[i](r1,z1);
	tpsi2 = wfuncs[i](r2,z2);
	psi += coeffs[i]*(tpsi1+tpsi2);
	hpsi+= coeffs[i]*((tpsi1+tpsi2)*energies[i]
			  - tpsi2*1.0/r1 - tpsi1*1.0/r2);
      }
      pp+=psi*psi*r;
      php+=psi*hpsi*r;
    }
  }
  return((php/pp+1/d+E_0)*NATURAL_ENERGY_UNIT);
}

int main(int argc, char **argv) {
  int i;
  if (argc!=WAVEFUNCTIONS+2) {
    printf("Usage: %s d c1 c2 ... c%d\n", argv[0], WAVEFUNCTIONS); exit(1);
  }

  d = fabs(atof(argv[1]));
  if (!d) { printf("d must not be 0!\n"); exit(2); }

  for (i=0; i<WAVEFUNCTIONS; i++) coeffs[i] = atof(argv[i+2]);

  printf("%f\n", expected_h());
  return(0);
}
